// 
// ASQView.cpp 
// 
#include "ASQView.h"
#include "TVolumenob.h"
#include "TSelectnob.h"
#include "TLamp.h"
#include "TTimer.h" 
#include "TOutput.h"
#include "TButton.h"
#include "TMenuField.h"
#include "TMidioutMenuField.h"
#include "TOutputMenuField.h"

#include <Box.h>
#include <PopUpMenu.h>
#include <Menu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <Rect.h>
#include <Message.h>
#include <MidiDefs.h>
#include <TextControl.h>

ASQView::ASQView( BRect frame, const char* name ) 
    :BView( frame, "sampleview", B_FOLLOW_ALL, B_WILL_DRAW ) 
{ 
	SetFont( be_plain_font ); 
	SetFontSize( 8 );
    rgb_color color = {255, 255, 255}; 
    rgb_color viewcolor = {60, 60, 60}; 
    SetViewColor( viewcolor ); 
    SetLowColor(viewcolor);
    SetHighColor( color ); 
    running = FALSE;
    stepping = FALSE;
    nowstepping= FALSE;
    now = 11;
    noofon=0;
	output=new TOutput(this, name);
} 

void ASQView::AttachedToWindow() 
{ 
	
	int i;
	BMessage* msg;
    rgb_color color = {255, 255, 255};
  	TButton* startButton;
    TButton* shotButton;
    
    timer=new TTimer(this,output,32,64,64);
    output->SetTarget(timer);
    BRect buttonFrame( 10, 20, 35, 45 );
    startButton = new TButton( buttonFrame, "startbutton", "", new BMessage( START_BUTTON_PRESSED )); 
    AddChild( startButton );
    buttonFrame.OffsetBy(0, 50);
    shotButton = new TButton( buttonFrame, "shotbutton", "", new BMessage( SHOT_BUTTON_PRESSED ),new BMessage( 'btup' )); 
    AddChild( shotButton );
   
    buttonFrame.OffsetBy(50, -50);
    BRect volumeFrame(60,20,90,50);
    temponob= new TVolumenob(volumeFrame, "volume", "",new BMessage('temp'),timer);
	volumeFrame.OffsetBy(0, 50);
	AddChild(temponob);
	temponob->SetValue(32);
	togulnob= new TSelectnob(volumeFrame, "togul", "",new BMessage('slct'),timer);
	AddChild(togulnob);
    togulnob->SetValue(2);
    buttonFrame.OffsetBy(30, 5);
    tempolamp= new TLamp(buttonFrame, "tempo", "");
    AddChild(tempolamp);
  
    BRect LampFrame(139,5,159,25);
    BRect volAFrame(140,40,160,60);
    BRect volBFrame(140,75,160,95);
    BRect volCFrame(140,110,160,130);
    
    
    for(i=0;i<16;i++){
    	msg= new BMessage('lamp');
    	msg->AddInt16("No.",i);
    	lamp[i]= new TLamp(LampFrame, "lamp", "",msg,this);
    	AddChild(lamp[i]);
    	msg= new BMessage('volm');
    	msg->AddInt16("CH",1);
    	msg->AddInt16("No.",i);
    	volA[i]= new TVolumenob(volAFrame, "vola", "",msg,output);
    	msg= new BMessage('volm');
    	msg->AddInt16("CH",2);
    	msg->AddInt16("No.",i);
    	volB[i]= new TVolumenob(volBFrame, "volb", "",msg,output);
    	msg= new BMessage('volm');
    	msg->AddInt16("CH",3);
    	msg->AddInt16("No.",i);
    	volC[i]= new TVolumenob(volCFrame, "volc", "",msg,output);
    	AddChild(volA[i]);
    	AddChild(volB[i]);
    	AddChild(volC[i]);
    	LampFrame.OffsetBy(31,0);
    	volAFrame.OffsetBy(31,0);
    	volBFrame.OffsetBy(31,0);
    	volCFrame.OffsetBy(31,0);
    }
    
    BRect volFrame(10,120,30,140);
    msg= new BMessage('ctra');
    msg->AddInt16("CH",1);
    portamentA= new TVolumenob(volFrame, "porA", "",msg,output);
    AddChild(portamentA);
    portamentA->SetValue(64);
    volFrame.OffsetBy(40,0);
    msg= new BMessage('ctrb');
    msg->AddInt16("CH",2);
    portamentB= new TVolumenob(volFrame, "porB", "",msg,output);
    AddChild(portamentB);
    portamentB->SetValue(64);
    volFrame.OffsetBy(40,0);
    duty= new TVolumenob(volFrame,"duty", "",new BMessage('duty'),timer);
    AddChild(duty);
    duty->SetValue(64);
    
    BPopUpMenu* menu;
    BMenuItem* item;
   	BRect fieldFrame( 5, 15, 190, 180 );
   	
   	BRect boxFrame(5,160,200,245);
    BBox* box;
    box= new BBox(boxFrame,"Output");
    box->SetLabel("Output");
    box->SetHighColor(color);
    box->SetLowColor(color);
    box->SetViewColor(color);
    AddChild(box);
    BMenuField* outA;
    char* strings[4];
    strings[1]="A";
    strings[2]="B";
    strings[3]="C";
    
    TMidioutMenuField* midiout;
    TOutputMenuField* outputA;
    
    for(i=1;i<4;i++){
    
    	menu = new BPopUpMenu("BSynth");
    	outputA = new TOutputMenuField(fieldFrame,strings[i], strings[i],menu, i);
    	outputA->SetHighColor( color );
    	outputA->SetDivider(15);
    	box->AddChild(outputA);
    	fieldFrame.OffsetBy(0,22);
   
    } 
    
    boxFrame.Set(205,160,365,245);
    box= new BBox(boxFrame,"midi connection");
    box->SetLabel("Midi Connection");
    box->SetHighColor(color);
    box->SetLowColor(color);
    box->SetViewColor(color);
    AddChild(box);
    
    fieldFrame.Set(5,15,155,180);
    menu = new BPopUpMenu("midiout");
	midiout = new TMidioutMenuField(fieldFrame, "midiout", "midiout", menu, 'mout');
	midiout->SetHighColor(color);
	midiout->SetDivider(40);
	box->AddChild(midiout);
	
	
	fieldFrame.Set( 5, 15, 115, 180 );
    strings[1]="Ctrl A";
    strings[2]="Ctrl B";
    boxFrame.Set(370,160,490,245);
    box= new BBox(boxFrame,"Controls");
    box->SetLabel("Controls");
    box->SetHighColor(color);
    box->SetLowColor(color);
    box->SetViewColor(color);
    AddChild(box);  

    for(i=1;i<3;i++){
    	menu = new BPopUpMenu("ctrl");
    	msg = new BMessage('ctrl');
    	msg->AddInt16("CH",i);
    	msg->AddInt16("data", 1);
    	item = new BMenuItem("Velosity",msg);
    	menu->AddItem(item);
    	msg = new BMessage('ctrl');
    	msg->AddInt16("CH",i);
    	msg->AddInt16("data",2);
    	item = new BMenuItem("Pitchbend",msg);
    	item->SetMarked(true);
    	menu->AddItem(item);
    	msg = new BMessage('ctrl');
    	msg->AddInt16("CH",i);
    	msg->AddInt16("data",3);
    	item = new BMenuItem("None",msg);
    	menu->AddItem(item);
    	msg = new BMessage('ctrl');
    	msg->AddInt16("CH",i);
    	msg->AddInt16("data",4);
    	
    	outA = new BMenuField(fieldFrame,strings[i], strings[i],menu);
    	outA->SetHighColor( color );
    	outA->SetDivider(40);
    	box->AddChild(outA);
    	fieldFrame.OffsetBy(0,22);
   	}   
   	
   		menu = new BPopUpMenu("reverb");
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_NONE);
    	item = new BMenuItem("None",msg);
    	menu->AddItem(item);
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_CLOSET);
    	item = new BMenuItem("Closet",msg);
    	menu->AddItem(item);
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_GARAGE);
    	item = new BMenuItem("Garage",msg);
    	menu->AddItem(item);
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_BALLROOM);
    	item = new BMenuItem("Ballroom",msg);
    	item->SetMarked(true);
    	menu->AddItem(item);
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_CAVERN);
    	item = new BMenuItem("Cavern",msg);
    	menu->AddItem(item);
    	msg = new BMessage('rvrb');
    	msg->AddInt16("Reverb",B_REVERB_DUNGEON);
    	item = new BMenuItem("Dungeon",msg);
    	menu->AddItem(item);
    	outA = new BMenuField(fieldFrame,"Reverb", "Reverb",menu);
    	outA->SetHighColor( color );
    	outA->SetDivider(40);
    	box->AddChild(outA);
    	
   		
   		boxFrame.Set(495,160,625,245);
    	box= new BBox(boxFrame,"Trigger Out");
    	box->SetLabel("Trigger Out");
    	box->SetHighColor(color);
    	box->SetLowColor(color);
    	box->SetViewColor(color);
    	AddChild(box); 
    	
    	fieldFrame.Set( 5, 15, 125, 180 );
    	msg =new BMessage('strg');
    	BPopUpMenu* trigmenu= new BPopUpMenu("Start Trigger");
    	TMenuField* starttrig= new TMenuField(fieldFrame,"Start Trigger", "Start Trig",trigmenu,'strg');
    	starttrig->SetHighColor( color );
    	starttrig->SetDivider(55);
    	box->AddChild(starttrig);
    	fieldFrame.OffsetBy(0,22);
    	msg =new BMessage('etrg');
    	trigmenu= new BPopUpMenu("End Trigger");
    	TMenuField* endtrig= new TMenuField(fieldFrame,"End Trigger", "End Trig",trigmenu,'etrg');
    	endtrig->SetHighColor( color );
    	endtrig->SetDivider(55);
    	box->AddChild(endtrig);
    	fieldFrame.OffsetBy(0,22);
    	msg =new BMessage('ctrg');
    	trigmenu= new BPopUpMenu("Clock Out");
    	TMenuField* clockout= new TMenuField(fieldFrame,"Clock Out", "Clock Out",trigmenu,'ctrg');
    	clockout->SetHighColor( color );
    	clockout->SetDivider(55);
    	box->AddChild(clockout);
    	
   	
    msg =new BMessage(('str0'));
    timer->PostMessage(msg);
    delete msg;
    endofnote=15;
    lamp[endofnote]->SetValue(2);
} 

void ASQView::MessageReceived(BMessage* message) 
{ 
   switch(message->what){
   		case 'puls':	tempolamp->SetValue(1);
   						break;
   		case 'half':	tempolamp->SetValue(0);
   						break;
   		case 'lpon':	if(noofon==endofnote) lamp[noofon]->SetValue(2);
   						else lamp[noofon]->SetValue(0);
   						noofon=message->FindInt16("No.");
   						lamp[noofon]->SetValue(1);
   						break;
   		case 'loff':	if(noofon==endofnote) lamp[noofon]->SetValue(2);
   						else lamp[noofon]->SetValue(0);
   						break;
   		case 'lamp':	lamp[endofnote]->SetValue(0);
   						endofnote=message->FindInt16("No.");
   						lamp[endofnote]->SetValue(2);
   						timer->PostMessage(message);
   						break;
   		case 'setv':	int32 v;
   						v=temponob->Value();
   						temponob->SetValue(v);
   						break;			
   		default:BView::MessageReceived(message);break;
   }
   
} 

void ASQView::Draw( BRect rect ) 
{ 
    SetFontSize(9);
    MovePenTo( 1, 15 ); 
    DrawString( "Start/Stop" ); 
    MovePenTo( 62, 15 ); 
    DrawString( "Clock" ); 
    MovePenTo( 13, 65 ); 
    DrawString( "Step" );
    MovePenTo( 40, 75 ); 
    SetFontSize(8);
    DrawString( "Step" );
    MovePenTo( 61, 65 ); 
    
    DrawString( "Single" );
    MovePenTo( 92, 75 ); 
    DrawString( "Repeat" );
    MovePenTo( 90, 115 ); 
    DrawString( "Duty" );
    MovePenTo( 7, 115 ); 
    DrawString( "Ctrl A" );
    MovePenTo( 47, 115 ); 
    DrawString( "Ctrl B" );
    MovePenTo( 127, 52 ); 
    SetFontSize(10);
    DrawString( "A" );
    MovePenTo( 127, 87 ); 
    DrawString( "B" );
    MovePenTo( 127, 122 ); 
    DrawString( "C" );
    SetFontSize(9);
    
   
    
} 

